----------------------------------------------------------------------------------
-- Create Date:    14:57:58 03/01/2012  
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity debounce is
    generic (
        N : natural := 22 -- (2**22)*10ns = 40ms   --cristal frequency is 100MHz for Spartan6
    );
    port (
        clk      : in  std_logic;
        reset    : in  std_logic;
        btn_sw   : in  std_logic;
        db_tick  : out std_logic; -- one-clock-cycle enable pulse
        db_level : out std_logic  -- stabilized output
    );
end debounce;

architecture Behavioral of debounce is
    type state_type is (zero, wait1, one, wait0); 
    signal state_reg, state_next : state_type; 
    signal q_reg, q_next         : unsigned(N-1 downto 0); 
    signal q_load, q_dec, q_zero : std_logic;
begin 
    -- FSMD state & data registers
    process(clk, reset)
    begin
        if rising_edge(clk) then
              if reset = '1' then
                    state_reg <= zero; 
                    q_reg <= (others => '0'); 
                else
                    state_reg <= state_next; 
                    q_reg <= q_next; 
                end if;
        end if;
    end process; 

    -- FSMD data path (counter) next_state logic
    q_next <= (others => '0') when (q_load = '1') else
              (q_reg - 1) when (q_dec = '1')      else
               q_reg; 
             
    q_zero <= '1' when (q_next = 0) else '0'; 

    -- FSMD control path next_state logic
    process(state_reg, btn_sw, q_zero)
    begin
        state_next <= state_reg; 
        q_load <= '0'; 
        q_dec <= '0'; 
    
        db_tick <= '0'; -- enable pulse asserted at the zero-to-one transition
    
        case state_reg is
            when zero => 
                db_level <= '0';
                
                if btn_sw = '1' then
                    q_load <= '1'; 
                    state_next <= wait1; 
                end if;
            
            when wait1 =>
                db_level <= '0';
                if btn_sw = '1' then
                    q_dec <= '1'; 
                
                    if q_zero = '1' then
                        db_tick <= '1'; 
                        state_next <= one; 
                    end if;
                else
                    state_next <= zero; 
                end if; 
            
            when one =>
                db_level <= '1';
                if btn_sw = '0' then
                    q_load <= '1'; 
                    state_next <= wait0; 
                end if; 
            
            when wait0 =>
                db_level <= '1';
                if (btn_sw = '0') then
                    q_dec <= '1'; 
                    
                    if (q_zero = '1') then 
                        state_next <= zero; 
                    end if; 
                else
                    state_next <= one; 
                end if;
        end case;
end process; 

end Behavioral;
