----------------------------------------------------------------------------------
-- Create Date:    17:14:40 11/08/2012 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.config.all; 

entity filter_transpose is
    Port (
        clk       : in  std_logic;
        reset     : in  std_logic;
        dowork    : in  std_logic; 
        num_ref   : in  integer range 0 to MAX_FBGS; 
        num_sen   : in  integer range 0 to MAX_FBGS; 
        init      : in  std_logic;
        amplitude : in  unsigned(AMPL_BITS-1 downto 0);
        adc_done  : in  std_logic;
        peak      : out std_logic_vector(RAMP_BITS-1 downto 0)
    );
end filter_transpose;

architecture Behavioral of filter_transpose is
    constant RAMP_LEN       : natural := 2**RAMP_BITS;
    constant SUM_AMPL_BITS  : natural := OUT_BITS + RAMP_BITS;
    
    --signals for FSMD
    type state_type is (idle, waiting, store, mean, outside, inside, peak_local); 
    signal state           : state_type;
    
    -- in this project, the spectral data is positive, signed, unsigned, std_logic_vector are the same
    signal sum_spectra     : unsigned(SUM_AMPL_BITS-1 downto 0);
    signal valve, valve_up : unsigned(OUT_BITS-1  downto 0);
    signal pamp_local      : unsigned(OUT_BITS-1  downto 0); 
    signal pt_local        : unsigned(RAMP_BITS-1 downto 0); 
    signal dif             : std_logic_vector(RAMP_BITS-1 downto 0);
    signal t               : natural range 0 to RAMP_LEN; -- counters
    signal num_peak: integer range 0 to MAX_FBGS; -- detect various FBGs
    signal peak_ref: unsigned(RAMP_BITS-1 downto 0); -- registrate the reference FBG peak
    
    --Memory for storing the specter data
    type mem_type is array(RAMP_LEN-1 downto 0) of std_logic_vector(OUT_BITS-1 downto 0);
    signal spectra : mem_type;
    
    -- used for fir_transpose instantiation
    signal din_aux: std_logic_vector(AMPL_BITS downto 0);  
    signal dout_aux: std_logic_vector(OUT_BITS -1 downto 0); 

begin
din_aux <= ("0" & std_logic_vector(amplitude) ) when (state = waiting or state = store) else (others => '0');
    
unit_filter: entity work.fir_transpose(behavioral)
port map(clk => clk, reset => reset, syn_tick => adc_done, din => din_aux, dout => dout_aux); 

process(clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' or dowork = '0' then
            state      <= idle; 
            t          <= 0; 
            sum_spectra<= (others => '0');
            valve      <= (others => '0'); 
            valve_up   <= (others => '0'); 
            pamp_local <= (others => '0'); 
            pt_local   <= (others => '0');
            dif        <= (others => '0'); 
            num_peak   <= 0; 
            peak_ref   <= (others => '0');
        else
            case state is
                when idle =>
                    if init = '1' then
                        t           <= 0;
                        state       <= waiting;
                        num_peak    <= 0; 
                        peak_ref    <= (others => '0'); 
                        sum_spectra <= (others => '0'); 
                    end if;
                    
                 when waiting => 
                    if adc_done = '1' then
                        if t = NLAT then
                            t <= 0; 
                            state <= store; 
                        else
                            t <= t + 1; 
                        end if; 
                     end if; 
                     
                  when store => 
                    if adc_done = '1' then
                        if (t = RAMP_LEN) then
                            t <= 0; 
                            state <= mean; 
                        else
                            spectra(t) <= dout_aux;
                            t <= t + 1; 
                        end if; 
                    end if; 
                    
                  when mean =>
                    if t = RAMP_LEN then
                        valve    <= sum_spectra(SUM_AMPL_BITS-1 downto RAMP_BITS) + to_unsigned(VALVE_RISE*50, OUT_BITS);
                        valve_up <= sum_spectra(SUM_AMPL_BITS-1 downto RAMP_BITS) + to_unsigned(PEAK_UP_THRES*50, OUT_BITS);
                        t        <= 0;
                        pt_local <= (others => '0');
                        state    <= outside;
                    else
                        sum_spectra  <= sum_spectra + unsigned(spectra(t)); 
                        t        <= t + 1;
                    end if;
     
                when outside =>
                    if t = RAMP_LEN then
                        if num_peak < num_sen then   -- wrong sensor number input detection
                            dif  <= (others => '1'); -- the difference between sensor and reference FBG should not be 4095
                        end if; 
                        state <= idle;
                    else
                        if unsigned( spectra(t) ) >= valve_up then
                            pamp_local <= valve;
                            state      <= inside;
                        else
                            t <= t + 1;
                        end if;
                    end if; 
                  
                when inside =>
                    if t = RAMP_LEN then
                        if num_peak < num_sen then   -- wrong sensor number input detection
                            dif  <= (others => '1'); -- the difference between sensor and reference FBG should not be 4095
                        end if; 
                        state <= idle;
                    else
                        if unsigned( spectra(t) ) <= valve then
                            state <= peak_local;
                            num_peak <= num_peak + 1; 
                        else
                            if unsigned( spectra(t) ) > pamp_local then
                                pamp_local  <= unsigned( spectra(t) );
                                pt_local    <= to_unsigned(t, RAMP_BITS);
                            end if;  
                            t <= t + 1;
                        end if;
                    end if; 
                    
                when peak_local =>
                    if num_ref = 0 then
                        peak_ref <= (others => '0'); 
                    else
                        if num_peak= num_ref then
                            peak_ref<= pt_local; 
                        end if; 
                    end if; 
                
                    if num_sen = 0 then
                        dif <= (others => '0'); 
                    else
                        if num_peak = num_sen then
                            dif <= std_logic_vector(pt_local - peak_ref); 
                        end if; 
                    end if; 
                    state <= outside; 
            end case; 
        end if; 
    end if; 
end process; 
    
peak <= std_logic_vector(dif);

end Behavioral;

