----------------------------------------------------------------------------------
-- Create Date:    18:42:45 05/11/2012
----------------------------------------------------------------------------------
library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity toplevel is
    port (
        clk        : in  std_logic;
        reset      : in  std_logic;
        sw         : in  std_logic;  -- choose configuration or operation
        sw1        : in  std_logic;  -- show configuration of frequency or mean level for DAC
        sw2        : in  std_logic;  -- show configuration of reference FBG
        sw3        : in  std_logic;  -- configure which data processing algorithm
        sw4        : in  std_logic;  -- configure 8 samples or only 1 sample in one step of ramp
        sw5        : in  std_logic;  -- display mode: 0: original. 1: average
        btn        : in  std_logic;  -- configure frequency  
        btn1       : in  std_logic;  -- configure mean level for DAC
        btn2       : in  std_logic;  -- configure which FBG works as reference FBG
        btn3       : in  std_logic;  -- configure which FBG works as sensor FBG
        
        adc_signal : in  std_logic_vector (AMPL_BITS-1 downto 0);
        wr         : out std_logic;
        sync       : out std_logic;
        out1       : out std_logic;
        out2       : out std_logic;
        sclk       : out std_logic;
        
        display    : out std_logic_vector (7 downto 0);
        anode      : out std_logic_vector (3 downto 0);
        
        tx         : out std_logic;
        rx         : in  std_logic
      
       
    );
end toplevel;

architecture Behavioral of toplevel is
    -- ramp generator and AD conversor
    signal init, mid, tick   : std_logic;
    signal tick_init         : std_logic;
    signal val               : std_logic_vector(RAMP_BITS-1 downto 0); 
    signal val_aux           : std_logic_vector(RAMP_BITS-1 downto 0); 
    signal mean              : std_logic_vector(RAMP_BITS-1 downto 0);
    signal enable            : std_logic;
    signal amplitude         : unsigned(AMPL_BITS-1 downto 0);
    signal adc_done          : std_logic;
    signal dac_done          : std_logic;
    signal count             : integer range 0 to MAX_COUNT;
    
    -- transmissor
    signal ampl_byte         : std_logic_vector(7 downto 0);
    signal wr_uart           : std_logic;
    signal tx_empty          : std_logic; 
    
    -- receiver
    signal flag              : std_logic; 
    signal clr_flag          : std_logic; 
    
    signal peak_centroid     : std_logic_vector(RAMP_BITS-1 downto 0); 
    signal peak_moving       : std_logic_vector(RAMP_BITS-1 downto 0); 
    signal peak_display      : std_logic_vector(RAMP_BITS-1 downto 0); -- for display
    signal do_centroid       : std_logic;
    signal do_filter         : std_logic;
    signal peak_mode         : std_logic; -- 0: centroid, 1: moving average

    signal count_freq        : integer range 0 to MAX_COUNT; 
    signal count_mean        : integer range 0 to 100; 
    signal count_fbgref      : integer range 0 to MAX_FBGS; 
    signal count_fbgsen      : integer range 0 to MAX_FBGS; 
    
    signal sw_level          : std_logic; 
    signal sw1_level         : std_logic; 
    signal sw2_level         : std_logic; 
    signal sw3_level         : std_logic; 
    signal sw4_level         : std_logic; 
    signal sw5_level         : std_logic; 
    signal btn_tick          : std_logic; 
    signal btn1_tick         : std_logic; 
    signal btn2_tick         : std_logic; 
    signal btn3_tick         : std_logic; 
    
begin
    
    ampl_byte   <= std_logic_vector(amplitude(AMPL_BITS-1 downto AMPL_BITS-8));   -- 8 msbs to be transmitted 
    do_centroid <= '1' when (enable = '1' and sw3_level = '0') else '0'; -- centroid
    do_filter   <= '1' when (enable = '1' and sw3_level = '1') else '0'; -- FIR filter
    tick_init   <= tick and init;
    
    --------------------------------------------------------------------------------------------------------
    -- peak display circuit
    peak_display_unit: entity work.peak_display(behavioral)
        port map(
            clk => clk, reset => reset, mode => sw3_level, mean => sw5_level, tick => tick_init,
            peak_centroid => peak_centroid, peak_moving => peak_moving, peak_output => peak_display); 
    --------------------------------------------------------------------------------------------------------
        
        
    --------------------------------------------------------------------------------------------------------
    --debouncing circuits     
    debouncing_circuits_unit: entity work.debouncing_circuits(behavioral)
        port map(
            clk => clk, reset => reset, sw => sw, sw1 => sw1, sw2 => sw2, sw3 => sw3, sw4 => sw4, sw5 => sw5,
            btn => btn, btn1 => btn1, btn2 => btn2, btn3 => btn3, sw_level => sw_level, 
            sw1_level => sw1_level, sw2_level => sw2_level, sw3_level => sw3_level, 
            sw4_level => sw4_level, sw5_level => sw5_level, btn_tick => btn_tick, btn1_tick => btn1_tick, btn2_tick => btn2_tick, 
            btn3_tick => btn3_tick); 
    --------------------------------------------------------------------------------------------------------
        
 
    --------------------------------------------------------------------------------------------------------
    --Configuration of system: reference FBG, sensor FBG, mean level of ramp, frequency of ramp
    btn_config_unit: entity work.btn_config(behavioral)
    port map(
        clk => clk, reset => reset, sw_level => sw_level, btn_tick => btn_tick, btn1_tick => btn1_tick, 
        btn2_tick => btn2_tick, btn3_tick => btn3_tick, count_fbgref => count_fbgref, 
        count_fbgsen => count_fbgsen, count_mean => count_mean, mean => mean, count => count
    ); 
    --------------------------------------------------------------------------------------------------------
 
 
    --------------------------------------------------------------------------------------------------------
    --Signal generator
    signal_generator_unit: entity work.signal_generator(behavioral)
    port map (
        clk => clk, reset => reset, count => count, sw => sw_level, val => val, enable => enable,
        counter => count_freq, init => init, mid => mid, tick => tick);
    --------------------------------------------------------------------------------------------------------
    
    
    --------------------------------------------------------------------------------------------------------
    -- look up table para calibrar o sinal de sintonia por meio de software
    lut_unit: entity work.lut(behavioral)
    port map(clk => clk, reset => reset, add => val, data => val_aux); 
    
    --------------------------------------------------------------------------------------------------------
    
    
    
    --------------------------------------------------------------------------------------------------------
    --Controll of D/A conversor and A/D conversor
    da2ctrl_unit: entity work.da2ctrl(behavioral)
    port map(clk => clk, reset => reset, data1 => val_aux, data2 => mean, start => tick,
        done => dac_done, out1 => out1, out2 => out2, sync => sync, sclk => sclk
    );
    
    adc_0820_unit: entity work.adc_0820(behavioral)     -- Atention, sw4_level should only be "0"
    port map (
        clk => clk, reset => reset, adc_signal => adc_signal, acq_tick => dac_done, mode => sw4_level,
        output => amplitude, adc_done => adc_done, wr => wr
    );
    --------------------------------------------------------------------------------------------------------
    
    
    
    --------------------------------------------------------------------------------------------------------
    --Data processing based on two algorithms: centroid and moving average
    centroid_unit: entity work.centroid(behavioral)
    port map(clk => clk, reset => reset, dowork => do_centroid, num_ref => count_fbgref, 
    num_sen => count_fbgsen,init => init, mid => mid, val => val, amplitude => amplitude, 
    adc_done => adc_done, peak => peak_centroid); 
             
--    moving_average_unit: entity work.moving_average(behavioral)
--    port map(clk => clk, reset => reset, dowork => do_filter, num_ref => count_fbgref, num_sen => count_fbgsen,
--             init => init, mid => mid, amplitude => amplitude, adc_done => adc_done, peak => peak_moving); 

    -- the simple filter (moving average) can be replaced by the transpose filter
             
    filter_transpose_unit: entity work.filter_transpose(behavioral)
    port map(clk => clk, reset => reset, dowork => do_filter, num_ref => count_fbgref, num_sen => count_fbgsen,
             init => init, amplitude => amplitude, adc_done => adc_done, peak => peak_moving);         
    --------------------------------------------------------------------------------------------------------
 
 
 
    --------------------------------------------------------------------------------------------------------
    --Controll of 7 segment displays
    display_freq_peak_unit: entity work.display_freq_peak(behavioral)
    port map(clk => clk, reset => reset, sw => sw_level, sw1 => sw1_level, sw2 => sw2_level,
            count_fbgref => count_fbgref, count_fbgsen => count_fbgsen, count_freq => count_freq, 
            count_mean => count_mean, peak => peak_display, anode => anode, display => display);
    --------------------------------------------------------------------------------------------------------
    
    
    
    --------------------------------------------------------------------------------------------------------
    --Communication 
    uart_unit: entity work.uart(behavioral)
    generic map(W => RAMP_BITS)
    port map (
        clk => clk, reset => reset, wr_uart => wr_uart, rx => rx,
        clr_flag => clr_flag, flag => flag, wr_data => ampl_byte, 
        tx_full => open,tx_empty => tx_empty, tx => tx );
    
    bridge_adc_uart_unit: entity work.bridge_adc_uart(behavioral)
    port map (
        clk => clk, reset => reset, init => init, mid => mid, enable => enable,
        tx_empty => tx_empty, adc_done => adc_done, tick => tick, flag => flag, 
        clr_flag => clr_flag, wr_uart => wr_uart);
    --------------------------------------------------------------------------------------------------------
    

   
end Behavioral;